@@ -3,13 +3,15 @@
 This module contains string utilities. Many of these routines can probably be
 replaced with better alternatives or even internal Python functions.
 
-Copyright 2015-2021 Lukas Kontenis
+Copyright 2015-2022 Lukas Kontenis
 Contact: dse.ssd@gmail.com
 """
 import string
+import pathlib
 import numpy as np
 
-from lklib.util import isnone, isarray, grep, msdo, warning_with_info
+from lkcom.util import isnone, isarray, grep, msdo, warning_with_info, \
+    handle_general_exception, find_closest
 
 
 def strip_nonnum(s):
@@ -32,21 +34,22 @@ def whitespace_to_underscore(s):
     return s.replace(' ', '_')
 
 
-def find_str_in_list(s, l, case_sensitive=False, exact_match=True, return_match_index=False):
-    """Compare a string to a list of strings."""
-    for ind, l1 in enumerate(l):
+def find_str_in_list(test_str, ref_str_arr, case_sensitive=False,
+                     exact_match=True, return_match_index=False):
+    """Find the index of a string that matches a list."""
+    for ind, ref_str in enumerate(ref_str_arr):
         if not case_sensitive:
-            s = s.lower()
-            l1 = l1.lower()
+            test_str = test_str.lower()
+            ref_str = ref_str.lower()
 
         if exact_match:
-            if s == l1:
+            if test_str == ref_str:
                 if return_match_index:
                     return ind
                 else:
                     return True
         else:
-            if l1.find(s) != -1 or s.find(l1) != -1:
+            if ref_str.find(test_str) != -1 or test_str.find(ref_str) != -1:
                 if return_match_index:
                     return ind
                 else:
@@ -58,28 +61,28 @@ def find_str_in_list(s, l, case_sensitive=False, exact_match=True, return_match_
         return False
 
 
-def search_list_in_string(s, l, case_sensitive=False):
-    for ind, l1 in enumerate(l):
-        if(case_sensitive):
-            if(s.find(l1) != -1):
+def search_list_in_string(test_str, ref_str_arr, case_sensitive=False):
+    for ind, ref_str in enumerate(ref_str_arr):
+        if case_sensitive:
+            if test_str.find(ref_str) != -1:
                 return ind
-        elif(s.lower().find(l1.lower()) != -1):
+        elif test_str.lower().find(test_str.lower()) != -1:
             return ind
 
     return None
 
 
-def compare_str_to_list(s, l, case_sensitive=False):
+def compare_str_to_list(test_str, ref_str_arr, case_sensitive=False):
     """Compare a string to a list of strings."""
     out = []
-    for l1 in l:
-        if(case_sensitive):
-            if(s == l1):
+    for ref_str in ref_str_arr:
+        if case_sensitive:
+            if test_str == ref_str:
                 out.append(True)
             else:
                 out.append(False)
         else:
-            if(s.lower() == l1.lower()):
+            if test_str.lower() == ref_str.lower():
                 out.append(True)
             else:
                 out.append(False)
@@ -120,12 +123,17 @@ def get_extension(fileName):
 
 def rem_extension(file_name):
     """Remove file extension."""
-    dot_pos = file_name.rfind('.')
+    return str(pathlib.Path(file_name).with_suffix(''))
 
-    if dot_pos != -1:
-        return file_name[:dot_pos]
-    else:
-        return file_name
+
+def get_bare_file_name(file_path):
+    """Get bare file name without slashes."""
+    return str(pathlib.Path(file_path).stem)
+
+
+def get_absolute_file_path(file_name):
+    """Get the absolute path of a file."""
+    return pathlib.Path(file_name).absolute()
 
 
 def change_extension(file_name, new_ext):
@@ -158,7 +166,13 @@ def string_with_units_to_float(s, units, unit_str=None):
     else:
         SI_fac = 1
 
-    return float(strip_alpha(s))*SI_fac
+    # Replace comma as decimal separator
+    s = s.replace(',', '.')
+
+    try:
+        return float(strip_alpha(s))*SI_fac
+    except Exception:
+        return None
 
 
 def parse_num_by_preffix(s, preffix):
@@ -179,13 +193,16 @@ def parse_num_by_preffix(s, preffix):
         return strip_nonnum(sel_chunk)
 
 
-def parse_num_by_units_name(s, units):
+def parse_num_by_units_name(s, units, split_token=None):
     """
     Parse a number of a complex string based on its units name. This should
     work for a unique number and unit pair with or without whitespace
     anywhere in any string.
     """
-    chunks = split_nonalpha(s)
+    if split_token is None:
+        chunks = split_nonalpha(s)
+    else:
+        chunks = s.split(split_token)
 
     sel_chunks, sel_chunk_inds = grep(chunks, units, return_inds=True)
 
@@ -207,40 +224,46 @@ def parse_num_by_units_name(s, units):
         return string_with_units_to_float(num_chunk, units, units_chunk)
 
 
-def multi_src_parse(val=None, src_arr=None, suffix=None, preffix=None):
+def multi_src_parse(val=None, src_arr=None, suffix=None, preffix=None,
+                    split_token=None):
     """Parse a suffixed value from multiple strings."""
-    if(not isnone(suffix) and not isarray(suffix)):
-        suffix = [suffix]
+    try:
+        if(not isnone(suffix) and not isarray(suffix)):
+            suffix = [suffix]
 
-    if(not isnone(preffix) and not isarray(preffix)):
-        preffix = [preffix]
+        if(not isnone(preffix) and not isarray(preffix)):
+            preffix = [preffix]
 
-    val_arr = [val]
-    for src in src_arr:
-        if(not isnone(src)):
-            if(not isnone(suffix)):
-                for sfx in suffix:
-                    val_arr.append(parse_num_by_units_name(src, sfx))
-            if(not isnone(preffix)):
-                for pfx in preffix:
-                    val_arr.append(parse_num_by_preffix(src, pfx))
+        val_arr = [val]
+        for src in src_arr:
+            if src is not None:
+                if suffix is not None:
+                    for sfx in suffix:
+                        val_arr.append(parse_num_by_units_name(
+                            src, sfx, split_token=split_token))
+                if preffix is not None:
+                    for pfx in preffix:
+                        val_arr.append(parse_num_by_preffix(src, pfx))
 
-    val_arr2 = []
-    for val2 in val_arr:
-        if(not(isnone(val2))):
-            val_arr2.append(val2)
-    val_arr = val_arr2
+        val_arr2 = []
+        for val2 in val_arr:
+            if(not(isnone(val2))):
+                val_arr2.append(val2)
+        val_arr = val_arr2
 
-    if(len(val_arr) == 0):
-        return None
+        if(len(val_arr) == 0):
+            return None
 
-    if(isnone(val)):
-        val = val_arr[0]
+        if(isnone(val)):
+            val = val_arr[0]
 
-    if(not all([val == v for v in val_arr])):
-        warning_with_info("Not all parsed values are consistent")
+        if(not all([val == v for v in val_arr])):
+            warning_with_info("Not all parsed values are consistent")
 
-    return val
+        return val
+    except Exception as excpt:
+        handle_general_exception("Failed to parse string", excpt)
+        return val
 
 
 def multi_src_match(output_str=None, src_arr=None, match_arr=None):
@@ -402,7 +425,8 @@ def get_si_suffix(num_order):
 
 def round_to_sig_fig(val, sig_fig):
     msdo_val = msdo(val)
-    return np.round(val / 10**msdo_val * 10**(sig_fig-1)) / 10**(sig_fig-1) * 10**msdo_val
+    return np.round(
+        val / 10**msdo_val * 10**(sig_fig-1)) / 10**(sig_fig-1) * 10**msdo_val
 
 
 def get_human_val_and_err_str(
@@ -456,21 +480,21 @@ def get_human_val_and_err_str(
 
         fmt_val = {
             "num_sig_fig": num_sig_fig, 'str_order': order,
-            "show_suffix": False, "suppress_suffix": kwargs.get('suppress_suffix'),
-            "fixed_width_sign": fixed_width_sign}
+            "show_suffix": False, "fixed_width_sign": fixed_width_sign,
+            "suppress_suffix": kwargs.get('suppress_suffix')}
 
         fmt_err = {
             "num_sig_fig": num_err_sig_fig, 'str_order': order,
-            "show_suffix": False, "suppress_suffix": kwargs.get('suppress_suffix'),
-            "fixed_width_sign": False}
+            "show_suffix": False, "fixed_width_sign": False,
+            "suppress_suffix": kwargs.get('suppress_suffix')}
 
         if not isnone(typ_val) and np.abs(val) < np.abs(err):
             # If value is close to typical, print it as '~typ'
             str = "~" + get_human_val_str(typ_val, num_sig_fig=2)
             approx = True
-        elif not isnone(max_abs_value) and np.abs(val) > max_abs_val:
+        elif not isnone(max_abs_value) and np.abs(val) > max_abs_value:
             # If value is greater than the maximum expected, print it as `>max'
-            str = ">" + get_human_val_str(max_abs_val, num_sig_fig=2)
+            str = ">" + get_human_val_str(max_abs_value, num_sig_fig=2)
             approx = True
         elif not isnone(min_abs_val) and np.abs(val) < min_abs_val:
             # If value is less than the minimum expected, print it as '~0'
@@ -500,9 +524,9 @@ def get_human_val_and_err_str(
 
 
 def get_human_val_str(
-        val, num_sig_fig=2, num_decimal_places=None, str_order=None, show_suffix=True,
+        val, num_sig_fig=2, num_decimal_places=None, str_order=None,
         space_before_suffix=False, is_time=False, fixed_width_sign=False,
-        fixed_str_len=None,
+        fixed_str_len=None,  show_suffix=True,
         min_abs_value=None, below_min_val_str='~0', trim_decimal_zeros=False,
         limit_decimal_places_to_num_sig_fig=False,
         limit_min_order_to_num_sig_fig=False, **kwargs):
@@ -570,7 +594,8 @@ def get_human_val_str(
 
                 # Time is different, because it's minutes and hours rather than
                 # kiloseconds. Override the multiplier and suffix if a number
-                # representing time is large, but use SI suffixes below a second.
+                # representing time is large, but use SI suffixes below a
+                # second.
                 if(is_time):
                     if(val > 3600):
                         mult = 3600
@@ -609,14 +634,14 @@ def get_human_val_str(
             # requested number of sig figs
             if nd1 >= num_sig_fig:
                 nd2 = 0
-                output_val = round(output_val/10**(nd1 - num_sig_fig)
-                                * 10**(nd1 - num_sig_fig))
+                output_val = round(
+                    output_val/10**(nd1 - num_sig_fig)*10**(nd1 - num_sig_fig))
 
                 # Calculate back the unscaled original value
                 val_check = output_val*mult
 
-                # If the suffix order of the rounded value has changed, call the
-                # function recursivelly to get the human-readable value
+                # If the suffix order of the rounded value has changed, call
+                # the function recursivelly to get the human-readable value
                 # TODO: This seems to be more complicated than it needs to be
                 # if(get_val_si_order(val, **kwargs) != str_order):
                 #    return get_human_readable_str(val_check, **kwargs)
@@ -626,8 +651,8 @@ def get_human_val_str(
 
                 # If the number of sig figs before the coma is zero, there may
                 # still be zeros after the comma which don't count as sig figs,
-                # for example the number 0.0023 is given with two sig figs, even
-                # though there are four digits after the comma
+                # for example the number 0.0023 is given with two sig figs,
+                # even though there are four digits after the comma
 
                 if 0 and val_order < 0:
                     # Not sure what this was handling but it doesn't work for
@@ -643,7 +668,6 @@ def get_human_val_str(
             # Build a human-readable value
             output_str = ("{:." + "{:d}f".format(int(nd2)) + "}").format(
                 output_val)
-            # HVal = "{:.f}".format(output_val)sprintf(sprintf('%%.%df', nd2), Val)
 
         if trim_decimal_zeros:
             output_str = str_trim_decimal_zeros(output_str)
@@ -678,9 +702,12 @@ def arr_summary_str(arr, **kwargs):
     min_val = np.nanmin(arr)
     max_val = np.nanmax(arr)
     if std_val < mean_val/1E6:
-        return "mean: " + get_human_val_str(mean_val, **kwargs) + ', all values the same'
+        return "mean: " + get_human_val_str(mean_val, **kwargs) + \
+               ", all values the same"
     else:
-        return "mean: " + get_human_val_str(mean_val, **kwargs) + ', min: ' + get_human_val_str(min_val, **kwargs) + ', max: ' + get_human_val_str(max_val, **kwargs)
+        return "mean: " + get_human_val_str(mean_val, **kwargs) + \
+               ", min: " + get_human_val_str(min_val, **kwargs) + \
+               ", max: " + get_human_val_str(max_val, **kwargs)
 
 
 def str_trim_decimal_zeros(s):
