@@ -3,16 +3,47 @@
 This module contains misc utilities. Many of these routines can probably be
 replaced with better alternatives or even internal Python functions.
 
-Copyright 2015-2021 Lukas Kontenis
+Copyright 2015-2022 Lukas Kontenis
 Contact: dse.ssd@gmail.com
 """
-import numpy as np
-import time
-
-from scipy.interpolate import interp1d
-
 import os
 import sys
+import time
+
+import numpy as np
+from matplotlib.colors import LinearSegmentedColormap
+from matplotlib.pyplot import get_cmap
+
+
+
+class MultiPrinter(object):
+    def __init__(self, *targets):
+        self.targets = targets
+
+    def write(self, obj):
+        for f in self.targets:
+            f.write(obj)
+            f.flush()
+
+    def flush(self):
+        for f in self.targets:
+            f.flush()
+
+
+class LogPrinter(object):
+    def __enter__(self):
+        self.log_file = open("output.log", "w")
+        self.stdout = sys.stdout
+        self.stderr = sys.stderr
+        print("Copying stdout and stderr stream to 'output.log'...")
+        sys.stdout = MultiPrinter(sys.stdout, self.log_file)
+        sys.stderr = MultiPrinter(sys.stderr, self.log_file)
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.log_file.close()
+        sys.stdout = self.stdout
+        sys.stderr = self.stderr
+        print("Restoring stdout and stderr streams to defaults...")
 
 
 def get_spec_line(line):
@@ -182,7 +213,7 @@ def grep(arr, substr, return_inds=False):
 
 def get_rsd(A):
     """Get the relative standard deviation of an array."""
-    return np.std(A)/np.mean(A)
+    return np.nanstd(A)/np.nanmean(A)
 
 
 def find_first(arr, val, equal_only=False, fail_with_nan=False):
@@ -217,7 +248,11 @@ def find_last(arr, val):
     Find the index of the last element in ar that is larger than or equal
     to val.
     """
-    return arr.size - find_first(np.flip(arr), val)
+    ind = find_first(np.flip(arr), val)
+    if ind is not None:
+        return arr.size - ind
+    else:
+        return None
 
 
 def round_to(par, thr=1E-3):
@@ -242,6 +277,26 @@ def find_closest(arr, val):
         return np.nanargmin((arr-val)**2)
 
 
+def interp(xval, xarr, yarr):
+    """1D interpolation for increasing or decreasing xarr."""
+    if (np.diff(xarr) > 0).all():
+        return np.interp(xval, xarr, yarr)
+    elif (np.diff(xarr) < 0).all():
+        return np.interp(xval, xarr[::-1], yarr[::-1])
+    else:
+        print("WARNING: xarr is nonmonotonous, interpolation impossible")
+        return None
+
+
+def cap_in_range(val, rng):
+    """Cap a value inside the given range."""
+    if val < rng[0]:
+        return rng[0]
+    if val > rng[1]:
+        return rng[1]
+    return val
+
+
 def extend_range(rng, frac):
     """
     Extend the given range by a given fraction.
@@ -533,6 +588,26 @@ def reduce_trace(X, Y, szr=None, Yred_method='mean'):
     return [Xr, Yr, Yr_sd]
 
 
+def hist_bin_edges_to_centers(binE):
+    """Calculate bin center positions from edge values."""
+    binC = np.ndarray(binE.size-1)
+    for ind in range(binE.size-1):
+        binC[ind] = (binE[ind] + binE[ind+1])/2
+
+    return binC
+
+
+def hist_bin_centers_to_edges(binC):
+    """Calculate bin edge positions from center values."""
+    binE = np.ndarray(len(binC)+1)
+    bin_step = np.mean(np.diff(binC))
+    binE[0] = binC[0] - (binC[1] - binC[0])/2
+    binE[-1] = binC[-1] + (binC[-1] - binC[-2])/2
+    for ind in range(1, len(binC)):
+        binE[ind] = binC[ind] + (binC[ind-1] - binC[ind])/2
+
+    return binE
+
 def bin_data(D, binsz):
     """
     Bin data into bins of a given size.
@@ -600,6 +675,56 @@ def bin_data_by_x(X, Y, binsz_X):
     return [Xb, Yb, Yb_sd]
 
 
+def bin_data_on_grid(xarr, yarr, xgrid, verbose=False):
+    """Bin data on a grid.
+
+    Useful for averaging data while resampling on a nonmonotonous grid, e.g.
+    when preparing data sampled on a linear x scale for display on a log scale.
+    """
+    ygrid = np.empty_like(xgrid)
+    if verbose:
+        print("Binning data on grid...")
+    for ind, xval in enumerate(xgrid):
+        if verbose and np.mod(ind, 10) == 0:
+            print("{:d}/{:d}".format(ind, len(xgrid)))
+        if ind > 0:
+            ind_from = find_closest(xarr, xgrid[ind-1])
+        else:
+            ind_from = 0
+        if ind < len(xgrid)-1:
+            ind_to = find_closest(xarr, xgrid[ind+1])
+        else:
+            ind_to = len(xarr)
+
+        ygrid[ind] = np.nanmean(yarr[ind_from:ind_to])
+
+    return ygrid
+
+
+def running_average(xarr, yarr, num_bins=None, window_sz=None):
+    """Perform a running average.
+
+    TODO: This uses the very slow bin_and_slice_data() algorithm.
+    """
+    if num_bins and window_sz:
+        raise ValueError("Specify either number of bins or window size")
+
+    if window_sz:
+        num_bins = int(len(xarr)/window_sz)
+
+    return bin_and_slice_data(xarr, yarr, num_bins=num_bins)[0:2]
+
+
+def columnize_array(arrs):
+    """Arrange 1D lists and arrays into columns in a 2D ndarray."""
+    num_rows = len(arrs[0])
+    arr2d = np.ndarray([num_rows, len(arrs)])
+    for ind, arr in enumerate(arrs):
+        arr2d[:, ind] = arr
+
+    return arr2d
+
+
 def bin_and_slice_data(X=None, Y=None, num_slcs=None, num_bins=None):
     """
     Combine the data into bins, slice each bin at equidistant percentile levels
@@ -909,7 +1034,7 @@ def get_color(name):
     return get_colour(name)
 
 
-def get_color_seq(names=None, seq_name=None):
+def get_color_seq(names=None, seq_name='Vega20'):
     """
     Get a color sequence for indexing.
     """
@@ -932,3 +1057,51 @@ def get_color_seq(names=None, seq_name=None):
         c.append(get_color(name))
 
     return c
+
+
+def get_colourmap(name):
+    """Get one of the default colourmaps by its name."""
+
+    if(name == "KBW_Nice"):
+        colors = [(0, 0, 0), (151/255, 185/255, 247/255), (1, 1, 1)]
+        return LinearSegmentedColormap.from_list(
+                name, colors, N=256)
+
+    if(name == "KPW_Nice"):
+        colors = [(0, 0, 0), (190/255, 70/255, 196/255), (1, 1, 1)]
+        return LinearSegmentedColormap.from_list(
+                name, colors, N=256)
+
+    if(name == "KGW_Nice"):
+        colors = [(0, 0, 0), (160/255, 255/255, 161/255), (1, 1, 1)]
+        return LinearSegmentedColormap.from_list(
+                name, colors, N=256)
+
+    if(name == "KOW_Nice"):
+        colors = [(0, 0, 0), (255/255, 180/255, 128/255), (1, 1, 1)]
+        return LinearSegmentedColormap.from_list(
+                name, colors, N=256)
+
+    if(name == "KRW_Nice"):
+        colors = [(0, 0, 0), (233/255, 64/255, 64/255), (1, 1, 1)]
+        return LinearSegmentedColormap.from_list(
+                name, colors, N=256)
+
+    if(name == "GYOR_Nice"):
+        colors = [(0.0, (130/255, 220/255, 132/255)),
+                  (0.25, (130/255, 220/255, 132/255)),
+                  (0.5, (255/255, 234/255, 151/255)),
+                  (0.75, (255/255, 180/255, 128/255)),
+                  (1.0, (255/255, 180/255, 185/255))]
+
+        return LinearSegmentedColormap.from_list(
+                name, colors, N=256)
+
+    if(name == 'gray_sat'):
+        cm = get_cmap('gray')
+        cm.set_over('r')
+        cm.set_under('b')
+
+        return cm
+
+    return get_cmap(name)
